/* esm.sh - esbuild bundle(htm@3.1.1/preact) es2022 production */
import { h as r } from "preact"; import { h as d, render as f, Component as h } from "preact"; import o from "./htm.mjs"; var p = o.bind(r); export { h as Component, d as h, p as html, f as render };
