
export const kInspectEnvValues = {
  providerModel: "INSPECT_EVAL_MODEL",
  logLevel: "INSPECT_LOG_LEVEL",
  logDir: "INSPECT_LOG_DIR",
  connections: "INSPECT_EVAL_MAX_CONNECTIONS",
  retries: "INSPECT_EVAL_MAX_RETRIES",
  timeout: "INSPECT_EVAL_TIMEOUT",
  modelBaseUrl: "INSPECT_MODE_BASE_URL",
};

export const kLogLevelEnv = "INSPECT_EVAL_MODEL";
